/*-------------------------------------------------------------------------
 *
 * pg_tde_utils.c
 *      Utility functions.
 *
 * IDENTIFICATION
 *    contrib/pg_tde/src/pg_tde_utils.c
 *
 *-------------------------------------------------------------------------
 */

#include "postgres.h"

#include "common/pg_tde_utils.h"
#include "pg_tde.h"

#ifndef FRONTEND
#include "fmgr.h"
#include "catalog/pg_class.h"
#include "access/pg_tde_tdemap.h"
#include "access/relation.h"
#include "utils/rel.h"

PG_FUNCTION_INFO_V1(pg_tde_is_encrypted);
Datum
pg_tde_is_encrypted(PG_FUNCTION_ARGS)
{
	Oid			relationOid = PG_GETARG_OID(0);
	LOCKMODE	lockmode = AccessShareLock;
	Relation	rel = relation_open(relationOid, lockmode);
	RelFileLocatorBackend rlocator = {.locator = rel->rd_locator,.backend = rel->rd_backend};
	InternalKey *key;

	if (!RELKIND_HAS_STORAGE(rel->rd_rel->relkind))
	{
		relation_close(rel, lockmode);
		PG_RETURN_NULL();
	}

	if (RelFileLocatorBackendIsTemp(rlocator) && !rel->rd_islocaltemp)
		ereport(ERROR,
				errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
				errmsg("we cannot check if temporary relations from other backends are encrypted"));

	key = GetSMGRRelationKey(rlocator);

	relation_close(rel, lockmode);

	PG_RETURN_BOOL(key != NULL);
}

#endif							/* !FRONTEND */

static char tde_data_dir[MAXPGPATH] = PG_TDE_DATA_DIR;

void
pg_tde_set_data_dir(const char *dir)
{
	Assert(dir != NULL);
	strlcpy(tde_data_dir, dir, sizeof(tde_data_dir));
}

const char *
pg_tde_get_data_dir(void)
{
	return tde_data_dir;
}
